(function() {
  var fs, getPath, inotify, inotifyFactory, isDirectorySync, isFileSync, path, unwatchDirectories, util, walkDirectoryTreeSync;
  var __indexOf = Array.prototype.indexOf || function(item) {
    for (var i = 0, l = this.length; i < l; i++) {
      if (this[i] === item) return i;
    }
    return -1;
  };
  fs = require('fs');
  inotifyFactory = require('inotify-plusplus');
  path = require('path');
  util = require('util');
  inotify = void 0;
  getPath = function(e) {
    return path.resolve(e.watch, e.name);
  };
  isDirectorySync = function(dir) {
    if (path.existsSync(dir)) {
      return fs.statSync(dir).isDirectory();
    } else {
      return false;
    }
  };
  isFileSync = function(file) {
    if (path.existsSync(file)) {
      return fs.statSync(file).isFile();
    } else {
      return false;
    }
  };
  unwatchDirectories = function(e, watchedDirectories) {
    var entry, entryEndPos, key, value, _results;
    entry = getPath(e);
    entryEndPos = entry.length - 1;
    _results = [];
    for (key in watchedDirectories) {
      value = watchedDirectories[key];
      _results.push(key.slice(0, (entryEndPos + 1) || 9e9) === entry ? (watchedDirectories[key](), delete watchedDirectories[key]) : void 0);
    }
    return _results;
  };
  walkDirectoryTreeSync = function(dir) {
    var addPathToList, pathList;
    pathList = [];
    addPathToList = function(entry) {
      var subEntry, _i, _len, _ref, _results;
      pathList.push(entry);
      if (isDirectorySync(entry)) {
        _ref = fs.readdirSync(entry);
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          subEntry = _ref[_i];
          _results.push(addPathToList(path.resolve(entry, subEntry)));
        }
        return _results;
      }
    };
    addPathToList(path.resolve(dir));
    return pathList;
  };
  exports.rmRecursiveSync = function(dir) {
    var traverseDirToRemove;
    traverseDirToRemove = function(removePath) {
      var entry, entryPath, _i, _len, _ref, _results;
      _ref = fs.readdirSync(removePath);
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        entry = _ref[_i];
        entryPath = path.resolve(removePath, entry);
        _results.push(isDirectorySync(entryPath) ? (traverseDirToRemove(entryPath), fs.rmdirSync(entryPath)) : fs.unlinkSync(entryPath));
      }
      return _results;
    };
    return traverseDirToRemove(path.resolve(dir));
  };
  exports.create = function(fileChangedCallback) {
    var addDirectoryToWatchList, watchedDirectories;
    watchedDirectories = [];
    addDirectoryToWatchList = function(dir) {
      var dirEntry, dirTree, _i, _len, _results;
      dir = path.resolve(dir);
      dirTree = walkDirectoryTreeSync(dir);
      dirTree = walkDirectoryTreeSync(dir);
      _results = [];
      for (_i = 0, _len = dirTree.length; _i < _len; _i++) {
        dirEntry = dirTree[_i];
        _results.push(isDirectorySync(dirEntry) && __indexOf.call(watchedDirectories, dirEntry) < 0 ? watchedDirectories[dirEntry] = inotify.watch({
          close_write: function(e) {
            var entry;
            entry = getPath(e);
            return fileChangedCallback(entry);
          },
          create: function(e) {
            var entry;
            entry = getPath(e);
            if (isDirectorySync(entry)) {
              return addDirectoryToWatchList(entry);
            }
          },
          "delete": function(e) {
            return unwatchDirectories(e, watchedDirectories);
          },
          moved_from: function(e) {
            return unwatchDirectories(e, watchedDirectories);
          },
          moved_to: function(e) {
            var entry;
            entry = getPath(e);
            if (isDirectorySync(entry)) {
              return addDirectoryToWatchList(entry);
            } else {
              return fileChangedCallback(entry);
            }
          }
        }, dirEntry) : void 0);
      }
      return _results;
    };
    return {
      get: function() {
        var key, value, _results;
        _results = [];
        for (key in watchedDirectories) {
          value = watchedDirectories[key];
          _results.push(key);
        }
        return _results;
      },
      watchDirectoryTreeSync: function(dir) {
        return addDirectoryToWatchList(dir);
      }
    };
  };
  exports.getMirrorPath = function(baseDirectoryPath, filePath, oldDirectoryName, newDirectoryName) {
    var oldPathPrefix, pathSuffix, startPos;
    oldPathPrefix = path.resolve(baseDirectoryPath, oldDirectoryName);
    startPos = oldPathPrefix.length;
    if (startPos > filePath.length) {
      throw 'getMirrorPath Error: baseDirectoryPath + oldDirectoryName should be shorter than filePath.';
    }
    if (filePath.slice(0, (startPos - 1 + 1) || 9e9) !== oldPathPrefix) {
      throw 'getMirrorPath Error: The prefix of filePath should match baseDirectoryPath + oldDirectoryName.';
    }
    pathSuffix = filePath.slice(startPos + 1, (filePath.length + 1) || 9e9);
    return path.resolve(baseDirectoryPath, newDirectoryName, pathSuffix);
  };
  exports.getRelativePath = function(basePath, longPath) {
    if (basePath[basePath.length - 1] !== '/') {
      basePath += '/';
    }
    if (basePath.length > longPath.length) {
      throw 'getPathSuffix Error: basePath should be shorter than or equal to longPath.';
    }
    if (longPath.slice(0, (basePath.length - 1 + 1) || 9e9) !== basePath) {
      throw 'getPathSuffix Error: The prefix of longPath should match basePath.';
    }
    return longPath.slice(basePath.length);
  };
  exports.isDirectorySync = isDirectorySync;
  exports.isFileSync = isFileSync;
  exports.setup = function(persist) {
    return inotify = inotifyFactory.create(persist);
  };
  exports.walkDirectoryTreeSync = walkDirectoryTreeSync;
}).call(this);
